@@ -26,270 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: h323pdu.h,v $
- * Revision 1.5  2008/09/27 06:15:38  shorne
- * BUG FIX: H323SignalPDU::Write to correctly handle NULL H323Connection Thx Nir Soffer
- *
- * Revision 1.4  2008/05/23 11:19:43  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.3  2007/10/19 19:53:44  shorne
- * ported latest Video updates in OpenH323 committed after h323plus initial fork thanks
- *  Robert
- *
- * Revision 1.2  2007/10/16 17:08:01  shorne
- * Allow cryptoTokens to be insert in the setup after receiving ARQ
- *
- * Revision 1.1  2007/08/06 20:50:49  shorne
- * First commit of h323plus
- *
- * Revision 1.75.2.1  2006/12/23 19:08:02  shorne
- * Plugin video codecs & sundry
- *
- * Revision 1.75  2006/07/17 09:52:57  shorne
- * Added more support URL alias Addresses
- *
- * Revision 1.74  2006/06/21 04:53:32  csoutheren
- * Tweaked H.245 version 13 updates
- *
- * Revision 1.73  2006/06/20 05:24:40  csoutheren
- * Additional tweaks for H.225v6
- *
- * Revision 1.72  2006/01/26 03:28:10  shorne
- * Caller Authentication added
- *
- * Revision 1.71  2006/01/20 00:32:24  csoutheren
- * First check-in of signalling aggregation code - incomplete and disabled by default
- *
- * Revision 1.70  2005/11/30 13:05:01  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 1.69  2005/06/30 01:57:39  csoutheren
- * Updated to H.245v11
- *
- * Revision 1.68  2005/03/04 03:21:20  csoutheren
- * Added local and remote addresses to all PDU logs to assist in debugging
- *
- * Revision 1.67  2005/01/24 00:10:48  csoutheren
- * Added function to set Q.931 info in PDU, thanks to Paul Nader
- *
- * Revision 1.66  2004/12/14 06:22:21  csoutheren
- * More OSP implementation
- *
- * Revision 1.65  2003/04/10 09:36:52  robertj
- * Added some more functions for converting to alias addresses.
- *
- * Revision 1.64  2003/04/01 03:11:01  robertj
- * Added function to get array of AliasAddress into PStringArray.
- *
- * Revision 1.63  2003/03/25 04:56:17  robertj
- * Fixed issues to do with multiple inheritence in transaction reply cache.
- *
- * Revision 1.62  2003/03/20 01:51:07  robertj
- * More abstraction of H.225 RAS and H.501 protocols transaction handling.
- *
- * Revision 1.61  2003/03/01 00:23:42  craigs
- * New PeerElement implementation
- *
- * Revision 1.60  2003/02/25 06:48:15  robertj
- * More work on PDU transaction abstraction.
- *
- * Revision 1.59  2003/02/21 05:28:39  craigs
- * Factored out code for user with peer elements
- *
- * Revision 1.58  2003/02/01 13:31:14  robertj
- * Changes to support CAT authentication in RAS.
- *
- * Revision 1.57  2002/11/28 04:41:44  robertj
- * Added support for RAS ServiceControlIndication command.
- *
- * Revision 1.56  2002/09/16 01:14:15  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 1.55  2002/09/03 06:19:37  robertj
- * Normalised the multi-include header prevention ifdef/define symbol.
- *
- * Revision 1.54  2002/08/12 05:38:21  robertj
- * Changes to the RAS subsystem to support ability to make requests to client
- *   from gkserver without causing bottlenecks and race conditions.
- *
- * Revision 1.53  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.52  2002/08/05 05:17:37  robertj
- * Fairly major modifications to support different authentication credentials
- *   in ARQ to the logged in ones on RRQ. For both client and server.
- * Various other H.235 authentication bugs and anomalies fixed on the way.
- *
- * Revision 1.51  2002/07/25 10:55:40  robertj
- * Changes to allow more granularity in PDU dumps, hex output increasing
- *   with increasing trace level.
- *
- * Revision 1.50  2002/07/11 07:04:12  robertj
- * Added build InfoRequest pdu type to RAS.
- *
- * Revision 1.49  2002/05/29 03:55:17  robertj
- * Added protocol version number checking infrastructure, primarily to improve
- *   interoperability with stacks that are unforgiving of new features.
- *
- * Revision 1.48  2002/05/29 00:03:15  robertj
- * Fixed unsolicited IRR support in gk client and server,
- *   including support for IACK and INAK.
- *
- * Revision 1.47  2002/05/07 03:18:12  robertj
- * Added application info (name/version etc) into registered endpoint data.
- *
- * Revision 1.46  2002/05/03 09:18:45  robertj
- * Added automatic retransmission of RAS responses to retried requests.
- *
- * Revision 1.45  2002/03/14 07:57:02  robertj
- * Added ability to specify alias type in H323SetAliasAddress, if not specified
- *   then defaults to previous behaviour, thanks Nils Bokerman.
- *
- * Revision 1.44  2001/12/15 07:09:56  robertj
- * Added functions to get E.164 address from alias address or addresses.
- *
- * Revision 1.43  2001/12/14 06:38:47  robertj
- * Broke out conversion of Q.850 and H.225 release complete codes to
- *   OpenH323 call end reasons enum.
- *
- * Revision 1.42  2001/12/13 10:56:04  robertj
- * Added build of request in progress pdu.
- *
- * Revision 1.41  2001/08/16 07:49:16  robertj
- * Changed the H.450 support to be more extensible. Protocol handlers
- *   are now in separate classes instead of all in H323Connection.
- *
- * Revision 1.40  2001/08/10 11:03:49  robertj
- * Major changes to H.235 support in RAS to support server.
- *
- * Revision 1.39  2001/08/06 07:44:52  robertj
- * Fixed problems with building without SSL
- *
- * Revision 1.38  2001/08/06 03:08:11  robertj
- * Fission of h323.h to h323ep.h & h323con.h, h323.h now just includes files.
- *
- * Revision 1.37  2001/06/14 06:25:13  robertj
- * Added further H.225 PDU build functions.
- * Moved some functionality from connection to PDU class.
- *
- * Revision 1.36  2001/06/14 00:45:19  robertj
- * Added extra parameters for Q.931 fields, thanks Rani Assaf
- *
- * Revision 1.35  2001/05/30 23:34:54  robertj
- * Added functions to send TCS=0 for transmitter side pause.
- *
- * Revision 1.34  2001/04/11 03:01:27  robertj
- * Added H.450.2 (call transfer), thanks a LOT to Graeme Reid & Norwood Systems
- *
- * Revision 1.33  2001/03/24 00:34:35  robertj
- * Added read/write hook functions so don't have to duplicate code in
- *    H323RasH235PDU descendant class of H323RasPDU.
- *
- * Revision 1.32  2001/03/21 04:52:40  robertj
- * Added H.235 security to gatekeepers, thanks Frbass Franz!
- *
- * Revision 1.31  2001/02/09 05:16:24  robertj
- * Added #pragma interface for GNU C++.
- *
- * Revision 1.30  2001/01/19 01:20:38  robertj
- * Added non-const function to get access to Q.931 PDU in H323SignalPDU.
- *
- * Revision 1.29  2000/10/04 05:59:09  robertj
- * Minor reorganisation of the H.245 secondary channel start up to make it simpler
- *    to override its behaviour.
- *
- * Revision 1.28  2000/09/25 06:47:54  robertj
- * Removed use of alias if there is no alias present, ie only have transport address.
- *
- * Revision 1.27  2000/09/22 01:35:02  robertj
- * Added support for handling LID's that only do symmetric codecs.
- *
- * Revision 1.26  2000/07/15 09:50:49  robertj
- * Changed adding of Q.931 party numbers to only occur in SETUP.
- *
- * Revision 1.25  2000/06/21 08:07:39  robertj
- * Added cause/reason to release complete PDU, where relevent.
- *
- * Revision 1.24  2000/05/23 11:32:27  robertj
- * Rewrite of capability table to combine 2 structures into one and move functionality into that class
- *    allowing some normalisation of usage across several applications.
- * Changed H323Connection so gets a copy of capabilities instead of using endponts, allows adjustments
- *    to be done depending on the remote client application.
- *
- * Revision 1.23  2000/05/08 14:07:26  robertj
- * Improved the provision and detection of calling and caller numbers, aliases and hostnames.
- *
- * Revision 1.22  2000/05/08 05:05:43  robertj
- * Fixed bug in H.245 close logical channel timeout, thanks XuPeili.
- *
- * Revision 1.21  2000/05/02 04:32:24  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.20  2000/04/10 20:39:30  robertj
- * Added support for more sophisticated DTMF and hook flash user indication.
- * Added function to extract E164 address from Q.931/H.225 PDU.
- *
- * Revision 1.19  2000/03/25 02:00:39  robertj
- * Added adjustable caller name on connection by connection basis.
- *
- * Revision 1.18  2000/03/21 01:22:01  robertj
- * Fixed incorrect call reference code being used in originated call.
- *
- * Revision 1.17  1999/12/11 02:20:58  robertj
- * Added ability to have multiple aliases on local endpoint.
- *
- * Revision 1.16  1999/09/10 03:36:48  robertj
- * Added simple Q.931 Status response to Q.931 Status Enquiry
- *
- * Revision 1.15  1999/08/31 12:34:18  robertj
- * Added gatekeeper support.
- *
- * Revision 1.14  1999/08/25 05:07:49  robertj
- * File fission (critical mass reached).
- *
- * Revision 1.13  1999/07/16 06:15:59  robertj
- * Corrected semantics for tunnelled master/slave determination in fast start.
- *
- * Revision 1.12  1999/07/16 02:15:30  robertj
- * Fixed more tunneling problems.
- *
- * Revision 1.11  1999/07/15 14:45:35  robertj
- * Added propagation of codec open error to shut down logical channel.
- * Fixed control channel start up bug introduced with tunnelling.
- *
- * Revision 1.10  1999/07/10 02:51:53  robertj
- * Added mutexing in H245 procedures.
- *
- * Revision 1.9  1999/07/09 06:09:49  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.8  1999/06/25 10:25:35  robertj
- * Added maintentance of callIdentifier variable in H.225 channel.
- *
- * Revision 1.7  1999/06/14 05:15:56  robertj
- * Changes for using RTP sessions correctly in H323 Logical Channel context
- *
- * Revision 1.6  1999/06/13 12:41:14  robertj
- * Implement logical channel transmitter.
- * Fixed H245 connect on receiving call.
- *
- * Revision 1.5  1999/06/09 05:26:20  robertj
- * Major restructuring of classes.
- *
- * Revision 1.4  1999/06/06 06:06:36  robertj
- * Changes for new ASN compiler and v2 protocol ASN files.
- *
- * Revision 1.3  1999/04/26 06:14:47  craigs
- * Initial implementation for RTP decoding and lots of stuff
- * As a whole, these changes are called "First Noise"
- *
- * Revision 1.2  1999/01/16 02:35:04  robertj
- * GNi compiler compatibility.
- *
- * Revision 1.1  1999/01/16 01:30:58  robertj
- * Initial revision
+ * $Id: h323pdu.h,v 1.10 2012/05/17 09:48:15 shorne Exp $
  *
  */
 
@@ -313,8 +50,8 @@
 class H225_RAS;
 
 
-#define H225_PROTOCOL_VERSION 6
-#define H245_PROTOCOL_VERSION 13
+#define H225_PROTOCOL_VERSION 7
+#define H245_PROTOCOL_VERSION 15
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -385,6 +122,12 @@ class H323SignalPDU : public H225_H323_UserInformation
 	  unsigned reason = H225_FacilityReason::e_undefinedReason ///< Reason for Facility
     );
 
+    /**Build a FACILITY message for Tunneling RAS Messages.
+      */
+    H225_Facility_UUIE * BuildRasFacility(
+        unsigned reason = H225_FacilityReason::e_featureSetUpdate    ///< Reason for Facility
+    );
+
     /**Build a PROGRESS message.
       */
     H225_Progress_UUIE & BuildProgress(
@@ -607,6 +350,9 @@ class H323ControlPDU : public H245_MultimediaSystemControlMessage
     H245_OpenLogicalChannelConfirm & BuildOpenLogicalChannelConfirm(
       unsigned channelNumber
     );
+    H245_IndicationMessage & BuildLogicalChannelActive(
+      unsigned channelNumber
+    );
     H245_CloseLogicalChannelAck & BuildCloseLogicalChannelAck(
       unsigned channelNumber
     );
@@ -722,6 +468,19 @@ class H323RasPDU : public H225_RasMessage, public H323TransactionPDU
 };
 
 
+/////////////////////////////////////////////////////////////////////////////
+// Language support
+
+void H323SetLanguage(const PString & lang, PASN_IA5String & asn);
+PBoolean H323SetLanguages(const PStringList & lang, H225_Setup_UUIE_language & asn);
+PBoolean H323SetLanguages(const PStringList & lang, H225_Connect_UUIE_language & asn);
+void H323GetLanguage(PStringList & lang, const PASN_IA5String & asn);
+PBoolean H323GetLanguages(PStringList & lang, const H225_Setup_UUIE_language & asn);
+PBoolean H323GetLanguages(PStringList & lang, const H225_Connect_UUIE_language & asn);
+
+PString H323GetDisplayName(const H225_ArrayOf_DisplayName & asn);
+PBoolean H323SetDisplayName(const PStringList & alias, const PStringList & lang, H225_ArrayOf_DisplayName & asn);
+
 /////////////////////////////////////////////////////////////////////////////
 
 void H323SetAliasAddresses(const H323TransportAddressArray & addresses, H225_ArrayOf_AliasAddress & aliases);
