@@ -33,7 +33,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Id: h460_std17.h,v 1.2 2011/07/30 04:11:41 shorne Exp $
+ * $Id: h460_std17.h,v 1.12 2012/12/10 17:56:08 shorne Exp $
  *
  *
  */
@@ -41,10 +41,9 @@
 #ifndef H_H460_FeatureStd17
 #define H_H460_FeatureStd17
 
-#if _MSC_VER > 1000
 #pragma once
-#endif // _MSC_VER > 1000
 
+#include <ptlib/plugin.h>
 #include <vector>
 #include <queue>
 
@@ -66,36 +65,18 @@ public:
 
     static PStringArray GetFeatureName() { return PStringArray("Std17"); };
     static PStringArray GetFeatureFriendlyName() { return PStringArray("NatTraversal-H.460.17"); };
-    static int GetPurpose()	{ return FeatureBase; };
+    static int GetPurpose();
 	static PStringArray GetIdentifier() { return PStringArray("17"); };
 
 	virtual PBoolean CommonFeature() { return isEnabled; }
 
+    static PBoolean IsEnabled()  { return isEnabled; }
+
     //////////////////////
     // Public Function  call..
-    PBoolean Initialise(const PString & remoteAddr);
+    virtual PBoolean Initialise(const PString & remoteAddr = PString(), PBoolean srv = true);
 
-    /////////////////////
-    // Registration messages
-    virtual PBoolean OnSendGatekeeperRequest(H225_FeatureDescriptor & pdu);
-    virtual void OnReceiveGatekeeperConfirm(const H225_FeatureDescriptor & pdu);
-
-    virtual PBoolean OnSendRegistrationRequest(H225_FeatureDescriptor & pdu);
-    virtual void OnReceiveRegistrationConfirm(const H225_FeatureDescriptor & pdu);
-
-    ////////////////////
-    // Signalling messages
-    virtual PBoolean OnSendSetup_UUIE(H225_FeatureDescriptor & pdu);
-    virtual void OnReceiveSetup_UUIE(const H225_FeatureDescriptor & pdu);
-
-    virtual PBoolean OnSendCallProceeding_UUIE(H225_FeatureDescriptor & pdu);
-    virtual void OnReceiveCallProceeding_UUIE(const H225_FeatureDescriptor & pdu);
-
-    virtual PBoolean OnSendAlerting_UUIE(H225_FeatureDescriptor & pdu);
-    virtual void OnReceiveAlerting_UUIE(const H225_FeatureDescriptor & pdu);
-
-    virtual PBoolean OnSendCallConnect_UUIE(H225_FeatureDescriptor & pdu);
-    virtual void OnReceiveCallConnect_UUIE(const H225_FeatureDescriptor & pdu);
+    H46017Handler * GetHandler() { return m_handler; }
 
 protected:
     PBoolean InitialiseTunnel(const H323TransportAddress & remoteAddr);
@@ -106,7 +87,7 @@ private:
     H323Connection * CON;
 
     H46017Handler * m_handler;
-    PBoolean isEnabled;
+    static PBoolean isEnabled;
 
 };
 
@@ -147,7 +128,6 @@ typedef std::priority_queue< std::pair<H323SignalPDU, h225Order::MessageHeader >
 ////////////////////////////////////////////////////////////////////////
 
 class H46017Handler;
-class PNatMethod_H46017;
 class H46017Transport  : public H323TransportTCP
 {
   PCLASSINFO(H46017Transport, H323TransportTCP);
@@ -162,6 +142,7 @@ class H46017Transport  : public H323TransportTCP
      */
     H46017Transport(
       H323EndPoint & endpoint,        /// H323 End Point object
+      PIPSocket::Address binding,     /// Bind Interface
 	  H46017Handler * feat		      /// Feature
     );
 
@@ -215,9 +196,9 @@ class H46017Transport  : public H323TransportTCP
     PBoolean ReadTunnel();
 
     PBoolean HandleControlPDU(const H323ControlPDU & pdu);
-
+#ifdef H323_H46026
     PBoolean PostFrame(unsigned sessionID,bool rtp,  const void * buf, PINDEX len);
-
+#endif
 	PBoolean isCall() { return isConnected; };
 
 	void ConnectionLost(PBoolean established);
@@ -242,10 +223,10 @@ class H46017Transport  : public H323TransportTCP
 
   protected:
     PBoolean ReadControl(const H323SignalPDU & msg);
+#ifdef H323_H46026
     PBoolean ReadMedia(const H323SignalPDU & msg);
-
     void GenerateFastUpdatePicture(int session);
-
+#endif
 	 PMutex connectionsMutex;
 	 PMutex WriteMutex;
 	 PMutex shutdownMutex;
@@ -265,11 +246,12 @@ class H46017Transport  : public H323TransportTCP
 	 PBoolean   remoteShutDown;
 	 PBoolean	closeTransport;
 
+     PString    callToken;
+
 };
 
 //////////////////////////////////////////////////////////////////////
 
-class PNatMethod_H46017;
 class H46017RasTransport;
 class H46017Handler : public PObject  
 {
@@ -303,6 +285,16 @@ public:
 
 	static H46017Transport * curtransport;
 
+#ifdef H323_H46026
+    /**Set Flag to say media tunneling
+      */
+    void SetH46026Tunnel(PBoolean tunnel);
+
+    /**Is Media Tunneling
+      */
+    PBoolean IsH46026Tunnel();
+#endif
+
 private:
 	H323EndPoint & ep;
 
@@ -315,13 +307,74 @@ private:
     PBoolean openTransport;
     PBoolean callEnded;
 
+#ifdef H323_H46026
+    PBoolean   m_h46026tunnel;
+#endif
+
 };
 
 ///////////////////////////////////////////////////////////////////////////
 
-class PNatMethod_H46017  : public PNatMethod
+#ifdef H323_H46026
+
+class H460_FeatureStd26 : public H460_FeatureStd 
 {
-	PCLASSINFO(PNatMethod_H46017,PNatMethod);
+    PCLASSINFO(H460_FeatureStd26,H460_FeatureStd);
+
+public:
+
+    H460_FeatureStd26();
+    virtual ~H460_FeatureStd26();
+
+    // Universal Declarations Every H460 Feature should have the following
+    virtual void AttachEndPoint(H323EndPoint * _ep);
+    virtual void AttachConnection(H323Connection * _con);
+
+    static PStringArray GetFeatureName() { return PStringArray("Std26"); };
+    static PStringArray GetFeatureFriendlyName() { return PStringArray("TCPTunneling-H.460.26"); };
+    static int GetPurpose();
+	static PStringArray GetIdentifier() { return PStringArray("26"); };
+
+	virtual PBoolean FeatureAdvertised(int mtype);
+	virtual PBoolean CommonFeature() { return isEnabled; }
+
+	// Messages
+    virtual PBoolean OnSendAdmissionRequest(H225_FeatureDescriptor & pdu);
+    virtual void OnReceiveAdmissionConfirm(const H225_FeatureDescriptor & pdu);
+
+    virtual PBoolean OnSendSetup_UUIE(H225_FeatureDescriptor & pdu);
+    virtual void OnReceiveSetup_UUIE(const H225_FeatureDescriptor & pdu);
+
+    virtual PBoolean OnSendCallProceeding_UUIE(H225_FeatureDescriptor & pdu);
+    virtual void OnReceiveCallProceeding_UUIE(const H225_FeatureDescriptor & pdu);
+
+    virtual PBoolean OnSendAlerting_UUIE(H225_FeatureDescriptor & pdu);
+    virtual void OnReceiveAlerting_UUIE(const H225_FeatureDescriptor & pdu);
+
+    virtual PBoolean OnSendCallConnect_UUIE(H225_FeatureDescriptor & pdu);
+    virtual void OnReceiveCallConnect_UUIE(const H225_FeatureDescriptor & pdu);
+
+    void AttachHandler(H46017Handler * m_handler);
+
+private:
+    H323EndPoint * EP;
+    H323Connection * CON;
+
+	H46017Handler * handler;			///< handler
+    PBoolean isEnabled;
+    static PBoolean isSupported;
+};
+
+// Need to declare for Factory Loader
+#ifndef _WIN32_WCE
+  PPLUGIN_STATIC_LOAD(Std26, H460_Feature);
+#endif
+
+//////////////////////////////////////////////////////////////////////////////
+
+class PNatMethod_H46026  : public PNatMethod
+{
+	PCLASSINFO(PNatMethod_H46026,PNatMethod);
 
 public:
 
@@ -329,17 +382,19 @@ public:
   //@{
 	/** Default Contructor
 	*/
-	PNatMethod_H46017();
+	PNatMethod_H46026();
 
 	/** Deconstructor
 	*/
-	~PNatMethod_H46017();
+	~PNatMethod_H46026();
   //@}
 
   /**@name General Functions */
   //@{
    void AttachEndPoint(H323EndPoint * ep);
 
+   void AttachHandler(H46017Handler * m_handler);
+
    virtual PBoolean GetExternalAddress(
       PIPSocket::Address & externalAddress, /// External address of router
       const PTimeInterval & maxAge = 1000   /// Maximum age for caching
@@ -380,24 +435,24 @@ public:
 		The Order of adding to the PNstStrategy determines which method
 		is used
   */
-   virtual bool IsAvailable(const PIPSocket::Address&) { return available; };
+   virtual bool IsAvailable(const PIPSocket::Address&) { return (handler && handler->IsH46026Tunnel()); };
 
-   void SetAvailable() { available = TRUE; };
+   void SetAvailable() {};
 
     /** Activate
 		Active/DeActivate the method on a call by call basis
 	 */
    virtual void Activate(bool act)  { active = act; }
 
-	/**  OpenSocket
-		Create a single UDP Socket 
-	*/
-   PBoolean OpenSocket(PUDPSocket & socket, PortInfo & portInfo, const PIPSocket::Address & binding) const;
-
-   static PStringArray GetNatMethodName() {  return PStringArray("H46017"); }
-
+#if PTLIB_VER > 2120
+   static PString GetNatMethodName() { return "H46026"; }
+   virtual PString GetName() const
+            { return GetNatMethodName(); }
+#else
+   static PStringArray GetNatMethodName() {  return PStringArray("H46026"); }
    virtual PString GetName() const
             { return GetNatMethodName()[0]; }
+#endif
 
    // All these are virtual and never used. 
     virtual bool GetServerAddress(
@@ -430,11 +485,12 @@ public:
     virtual bool CreateSocket(BYTE component,PUDPSocket * & socket,
             const PIPSocket::Address & binding = PIPSocket::GetDefaultIpAny(),WORD localPort = 0)  { return false; }
     virtual void SetCredentials(const PString &, const PString &, const PString &) {}
+protected:
+    virtual NatTypes InternalGetNatType(bool, const PTimeInterval &) { return UnknownNat; }
 #endif
 
 
 protected:
-	PBoolean available;					///< Whether this NAT Method is available for call
 	PBoolean active;					///< Whether the method is active for call
 	H46017Handler * handler;			///< handler
 
@@ -442,27 +498,27 @@ protected:
 
 
 #ifndef _WIN32_WCE
-PPLUGIN_STATIC_LOAD(H46017, PNatMethod);
+PPLUGIN_STATIC_LOAD(H46026, PNatMethod);
 #endif
 
 typedef std::queue<PBYTEArray*> RTPQueue;
 
-class H46017A_UDPFrame;
-class H46017UDPSocket : public PUDPSocket
+class H46026_UDPFrame;
+class H46026UDPSocket : public PUDPSocket
 {
-  PCLASSINFO(H46017UDPSocket, PUDPSocket);
+  PCLASSINFO(H46026UDPSocket, PUDPSocket);
   public:
   /**@name Construction/Deconstructor */
   //@{
 	/** create a UDP Socket Fully Nat Supported
 		ready for H323plus to Call.
 	*/
-    H46017UDPSocket(H46017Handler & _handler, H323Connection::SessionInformation * info, bool _rtpSocket);
+    H46026UDPSocket(H46017Handler & _handler, H323Connection::SessionInformation * info, bool _rtpSocket);
 
 	/** Deconstructor to reallocate Socket and remove any exiting
 		allocated NAT ports, 
 	*/
-	~H46017UDPSocket();
+	~H46026UDPSocket();
 
    //@}
 
@@ -499,24 +555,16 @@ class H46017UDPSocket : public PUDPSocket
     );
 
     PBoolean WriteBuffer(const void * buf, PINDEX len);
+    PBoolean DoPseudoRead(int & selectStatus);
     PBoolean BuildTunnelMediaPacket(const void * buf, PINDEX len);
 
     void GetLocalAddress(H245_TransportAddress & add);
+    PBoolean GetLocalAddress(Address & addr, WORD & port);
+
     void SetRemoteAddress(const H245_TransportAddress & add);
 
    //@}
 
-#if PTLIB_VER >= 2110
-    virtual PString GetServer() const { return PString(); }
-    virtual bool GetServerAddress(PIPSocketAddressAndPort & ) const { return false; }
-    virtual NatTypes GetNatType(bool) { return UnknownNat; }
-    virtual NatTypes GetNatType(const PTimeInterval &) { return UnknownNat; }
-    virtual bool SetServer(const PString &) { return false; }
-    virtual bool Open(const PIPSocket::Address &) { return false; }
-    virtual bool CreateSocket(BYTE,PUDPSocket * &, const PIPSocket::Address,WORD)  { return false; }
-    virtual void SetCredentials(const PString &, const PString &, const PString &) {}
-#endif
-
 
 private:
 	H46017Transport * m_transport;
@@ -526,13 +574,14 @@ private:
     // tunnel data containers
     int m_frameCount;
     int m_payloadSize;
-    H46017A_UDPFrame * m_frame;
+    H46026_UDPFrame * m_frame;
     H323SignalPDU msg;
 
     RTPQueue rtpQueue;
     PMutex writeMutex;
     PBoolean shutDown;
 };
+#endif // H323_H46026
 
 //////////////////////////////////////////////////////////////////////
 
@@ -548,12 +597,25 @@ class H46017RasTransport : public H323TransportUDP
     );
     ~H46017RasTransport();
 
+    virtual H323TransportAddress GetLocalAddress() const;
+
+    virtual void SetUpTransportPDU(
+      H225_TransportAddress & pdu,
+      PBoolean localTsap,
+      H323Connection * connection = NULL
+    ) const;
+
     virtual PBoolean SetRemoteAddress(
       const H323TransportAddress & address
     );
 
     virtual PBoolean Connect();
 
+    /**Close the channel.
+      */
+    virtual PBoolean Close();
+
+
     PBoolean ReceivedPDU(
       const PBYTEArray & pdu  ///<  PDU read from Tunnel
     );
@@ -572,7 +634,9 @@ class H46017RasTransport : public H323TransportUDP
       const H323TransportAddress & address  ///<  Address of gatekeeper (if present)
     );
 
-    virtual H323TransportAddress GetLocalAddress() const;
+    virtual PBoolean IsRASTunnelled();
+
+    virtual PChannel::Errors GetErrorCode(ErrorGroup group = NumErrorGroups) const;
 
   protected:
     H46017Handler * m_handler;
@@ -581,6 +645,7 @@ class H46017RasTransport : public H323TransportUDP
     PSyncPoint  msgRecd;
     PBYTEArray  recdpdu;
 
+    PBoolean    shutdown;
 
 };
 
