@@ -26,259 +26,7 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Log: rtp.h,v $
- * Revision 1.15  2011/05/09 07:35:47  shorne
- * updates for changes in PTLIB v2.11. The replacement of PDICTIONARY with H323Dictionary. Updates for the changes in the PNatMethod Class. Alteration in PFactory::Create, Remove redundent code and change change logging
- *
- * Revision 1.14  2011/01/12 13:08:38  shorne
- * H.239 Clunge for channelID=0 and set Channelid in OLCack
- *
- * Revision 1.13  2010/09/19 05:53:28  shorne
- * Added NAT Support for Unidirection media channels (H.239) fix to set random timestamp rather than  zero. Fix the timestamp on the marker frame.
- *
- * Revision 1.12  2010/08/26 15:12:39  shorne
- * Major H.239 upgrade. Special thx again to Marek Domaracky and Igor Pavlov
- *
- * Revision 1.11  2010/06/06 14:53:26  shorne
- * Added AVSync support, Aspect Ratio management, flow Control, Video 90k clock, fixes for wideband codecs and generic audio capabilities
- *
- * Revision 1.10  2010/05/03 03:56:34  shorne
- * Improved collection of sender reports including identifying session they originated from
- *
- * Revision 1.9  2010/04/12 21:39:54  willamowius
- * give application access to RTP sender reports
- *
- * Revision 1.8  2009/11/29 23:31:05  shorne
- * BUG FIX : completely disable H.460 support if remote does not support it.
- *
- * Revision 1.7  2009/07/09 15:11:12  shorne
- * Simplfied and standardised compiler directives
- *
- * Revision 1.6  2009/06/28 01:41:52  shorne
- * Replaced P_HAS_QOS with P_QOS (depreciated in PTLib)
- *
- * Revision 1.5  2008/05/23 11:20:01  willamowius
- * switch BOOL to PBoolean to be able to compile with Ptlib 2.2.x
- *
- * Revision 1.4  2008/02/06 02:52:59  shorne
- * Added support for Standards based NAT Traversal
- *
- * Revision 1.3  2008/01/22 01:10:32  shorne
- * Fix H.224 opening unidirectional channel
- *
- * Revision 1.2  2007/10/16 17:03:53  shorne
- * Qos capability negotiation
- *
- * Revision 1.1  2007/08/06 20:50:50  shorne
- * First commit of h323plus
- *
- * Revision 1.60.2.4  2007/07/23 21:47:10  shorne
- * Added QoS GK Reporting
- *
- * Revision 1.60.2.3  2007/03/24 23:39:42  shorne
- * More H.239 work
- *
- * Revision 1.60.2.2  2007/02/17 15:26:20  shorne
- * Support static memory allocation of RTP_DataFrame for Plugin Video Support
- *
- * Revision 1.60.2.1  2006/12/23 19:08:02  shorne
- * Plugin video codecs & sundry
- *
- * Revision 1.60  2006/01/26 03:35:03  shorne
- * Added more NAT traversal support
- *
- * Revision 1.59  2006/01/18 07:46:08  csoutheren
- * Initial version of RTP aggregation (disabled by default)
- *
- * Revision 1.58  2005/11/30 13:05:01  csoutheren
- * Changed tags for Doxygen
- *
- * Revision 1.57  2005/11/21 21:06:23  shorne
- * Added GQoS switches
- *
- * Revision 1.56  2005/08/27 02:06:00  csoutheren
- * Keep track of time first RTP packet is received
- *
- * Revision 1.55  2005/01/03 14:03:20  csoutheren
- * Added new configure options and ability to disable/enable modules
- *
- * Revision 1.54  2005/01/03 06:25:52  csoutheren
- * Added extensive support for disabling code modules at compile time
- *
- * Revision 1.53  2004/04/22 12:38:02  rjongbloed
- * Removed the RTP QoS class if there is no QoS support in build,
- *   thanks Nick Hoath, ISDN Communications Ltd.
- *
- * Revision 1.52  2004/04/07 05:31:42  csoutheren
- * Added ability to receive calls from endpoints behind NAT firewalls
- *
- * Revision 1.51  2003/10/27 06:03:39  csoutheren
- * Added support for QoS
- *   Thanks to Henry Harrison of AliceStreet
- *
- * Revision 1.50  2003/10/09 09:47:45  csoutheren
- * Fixed problem with re-opening RTP half-channels under unusual
- * circumstances. Thanks to Damien Sandras
- *
- * Revision 1.49  2003/05/14 13:46:39  rjongbloed
- * Removed hack of using special payload type for H.263 for a method which
- *   would be less prone to failure in the future.
- *
- * Revision 1.48  2003/05/02 04:57:43  robertj
- * Added header extension support to RTP data frame class.
- *
- * Revision 1.47  2003/05/02 04:21:53  craigs
- * Added hacked OpalH263 codec
- *
- * Revision 1.46  2003/02/07 00:30:41  robertj
- * Changes for bizarre usage of RTP code outside of scope of H.323 specs.
- *
- * Revision 1.45  2003/02/05 06:32:08  robertj
- * Fixed non-stun symmetric NAT support recently broken.
- *
- * Revision 1.44  2003/02/04 07:06:41  robertj
- * Added STUN support.
- *
- * Revision 1.43  2002/11/19 01:48:15  robertj
- * Allowed get/set of canonical anme and tool name.
- *
- * Revision 1.42  2002/11/12 22:10:48  robertj
- * Updated documentation.
- *
- * Revision 1.41  2002/10/31 00:33:29  robertj
- * Enhanced jitter buffer system so operates dynamically between minimum and
- *   maximum values. Altered API to assure app writers note the change!
- *
- * Revision 1.40  2002/09/26 04:01:58  robertj
- * Fixed calculation of fraction of packets lost in RR, thanks Awais Ali
- *
- * Revision 1.39  2002/09/16 01:14:15  robertj
- * Added #define so can select if #pragma interface/implementation is used on
- *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
- *
- * Revision 1.38  2002/09/03 05:47:02  robertj
- * Normalised the multi-include header prevention ifdef/define symbol.
- * Added copy constructor/operator for session manager.
- *
- * Revision 1.37  2002/08/05 10:03:47  robertj
- * Cosmetic changes to normalise the usage of pragma interface/implementation.
- *
- * Revision 1.36  2002/05/28 02:37:37  robertj
- * Fixed reading data out of RTCP compound statements.
- *
- * Revision 1.35  2002/05/02 05:58:24  robertj
- * Changed the mechanism for sending RTCP reports so that they will continue
- *   to be sent regardless of if there is any actual data traffic.
- * Added support for compound RTCP statements for sender and receiver reports.
- *
- * Revision 1.34  2002/02/09 02:33:37  robertj
- * Improved payload type docuemntation and added Cisco CN.
- *
- * Revision 1.33  2002/01/22 07:08:26  robertj
- * Added IllegalPayloadType enum as need marker for none set
- *   and MaxPayloadType is a legal value.
- *
- * Revision 1.32  2001/11/09 05:39:54  craigs
- * Added initial T.38 support thanks to Adam Lazur
- *
- * Revision 1.31  2001/09/11 00:21:21  robertj
- * Fixed missing stack sizes in endpoint for cleaner thread and jitter thread.
- *
- * Revision 1.30  2001/07/06 06:32:22  robertj
- * Added flag and checks for RTP data having specific SSRC.
- * Changed transmitter IP address check so is based on first received
- *    PDU instead of expecting it to come from the host we are sending to.
- *
- * Revision 1.29  2001/06/04 11:37:48  robertj
- * Added thread safe enumeration functions of RTP sessions.
- * Added member access functions to UDP based RTP sessions.
- *
- * Revision 1.28  2001/04/24 06:15:50  robertj
- * Added work around for strange Cisco bug which suddenly starts sending
- *   RTP packets beginning at a difference sequence number base.
- *
- * Revision 1.27  2001/04/02 23:58:23  robertj
- * Added jitter calculation to RTP session.
- * Added trace of statistics.
- *
- * Revision 1.26  2001/02/09 05:16:24  robertj
- * Added #pragma interface for GNU C++.
- *
- * Revision 1.25  2000/12/18 08:58:30  craigs
- * Added ability set ports
- *
- * Revision 1.24  2000/09/25 01:44:31  robertj
- * Fixed possible race condition on shutdown of RTP session with jitter buffer.
- *
- * Revision 1.23  2000/09/21 02:06:06  craigs
- * Added handling for endpoints that return conformant, but useless, RTP address
- * and port numbers
- *
- * Revision 1.22  2000/05/23 12:57:28  robertj
- * Added ability to change IP Type Of Service code from applications.
- *
- * Revision 1.21  2000/05/18 11:53:34  robertj
- * Changes to support doc++ documentation generation.
- *
- * Revision 1.20  2000/05/04 11:49:21  robertj
- * Added Packets Too Late statistics, requiring major rearrangement of jitter buffer code.
- *
- * Revision 1.19  2000/05/02 04:32:25  robertj
- * Fixed copyright notice comment.
- *
- * Revision 1.18  2000/05/01 01:01:24  robertj
- * Added flag for what to do with out of orer packets (use if jitter, don't if not).
- *
- * Revision 1.17  2000/04/30 03:55:18  robertj
- * Improved the RTCP messages, epecially reports
- *
- * Revision 1.16  2000/04/10 17:39:21  robertj
- * Fixed debug output of RTP payload types to allow for unknown numbers.
- *
- * Revision 1.15  2000/04/05 03:17:31  robertj
- * Added more RTP statistics gathering and H.245 round trip delay calculation.
- *
- * Revision 1.14  2000/03/23 02:55:18  robertj
- * Added sending of SDES control packets.
- *
- * Revision 1.13  2000/03/20 20:54:04  robertj
- * Fixed problem with being able to reopen for reading an RTP_Session (Cisco compatibilty)
- *
- * Revision 1.12  2000/02/29 13:00:13  robertj
- * Added extra statistic display for RTP packets out of order.
- *
- * Revision 1.11  1999/12/30 09:14:31  robertj
- * Changed payload type functions to use enum.
- *
- * Revision 1.10  1999/12/23 23:02:35  robertj
- * File reorganision for separating RTP from H.323 and creation of LID for VPB support.
- *
- * Revision 1.9  1999/11/20 05:35:57  robertj
- * Fixed possibly I/O block in RTP read loops.
- *
- * Revision 1.8  1999/11/19 09:17:15  robertj
- * Fixed problems with aycnhronous shut down of logical channels.
- *
- * Revision 1.7  1999/11/14 11:41:18  robertj
- * Added access functions to RTP statistics.
- *
- * Revision 1.6  1999/09/08 04:05:48  robertj
- * Added support for video capabilities & codec, still needs the actual codec itself!
- *
- * Revision 1.5  1999/08/31 12:34:18  robertj
- * Added gatekeeper support.
- *
- * Revision 1.4  1999/07/13 09:53:24  robertj
- * Fixed some problems with jitter buffer and added more debugging.
- *
- * Revision 1.3  1999/07/09 06:09:49  robertj
- * Major implementation. An ENORMOUS amount of stuff added everywhere.
- *
- * Revision 1.2  1999/06/22 13:49:40  robertj
- * Added GSM support and further RTP protocol enhancements.
- *
- * Revision 1.1  1999/06/14 06:12:25  robertj
- * Changes for using RTP sessions correctly in H323 Logical Channel context
+ * $ Id $
  *
  */
 
@@ -380,6 +128,9 @@ class RTP_DataFrame : public PBYTEArray
     PBoolean GetExtension() const   { return (theArray[0]&0x10) != 0; }
     void SetExtension(PBoolean ext);
 
+    PBoolean GetPadding()  { return (theArray[0]&0x20) != 0; }
+    void SetPadding(PBoolean padding);
+
     PBoolean GetMarker() const { return (theArray[1]&0x80) != 0; }
     void SetMarker(PBoolean m);
 
@@ -387,6 +138,7 @@ class RTP_DataFrame : public PBYTEArray
     void         SetPayloadType(PayloadTypes t);
 
     WORD GetSequenceNumber() const { return *(PUInt16b *)&theArray[2]; }
+    BYTE * GetSequenceNumberPtr() const;
     void SetSequenceNumber(WORD n) { *(PUInt16b *)&theArray[2] = n; }
 
     DWORD GetTimestamp() const  { return *(PUInt32b *)&theArray[4]; }
@@ -411,6 +163,8 @@ class RTP_DataFrame : public PBYTEArray
     PBoolean   SetPayloadSize(PINDEX sz);
     BYTE * GetPayloadPtr()     const { return (BYTE *)(theArray+GetHeaderSize()); }
 
+    PBoolean IsValid() const;
+
   protected:
     PINDEX payloadSize;
 
@@ -419,7 +173,45 @@ class RTP_DataFrame : public PBYTEArray
 #endif
 };
 
-PLIST(RTP_DataFrameList, RTP_DataFrame);
+H323LIST(RTP_DataFrameList, RTP_DataFrame);
+
+
+/**This class is for encapsulating the Multiplexing of RTP Data.
+ */
+
+class RTP_MultiDataFrame : public PBYTEArray
+{
+  PCLASSINFO(RTP_MultiDataFrame, PBYTEArray);
+
+ public:
+
+  RTP_MultiDataFrame(
+      BYTE const * buffer,   ///< Pointer to an array of BYTEs.
+      PINDEX length          ///< Number of elements pointed to by \p buffer.
+  );
+
+  RTP_MultiDataFrame(
+      DWORD id,              ///< Mulitplex ID
+      const BYTE * buffer,   ///< Pointer to an array of BYTEs.
+      PINDEX rtplen          ///< Number of elements pointed to by \p buffer.
+  );
+
+  RTP_MultiDataFrame(
+      PINDEX rtplen          ///< Length of RTP Frame
+  );
+
+  int   GetMultiHeaderSize() const;
+  DWORD GetMultiplexID() const;
+  void SetMultiplexID(DWORD id);
+  void GetRTPPayload(RTP_DataFrame & frame) const;
+  void SetRTPPayload(RTP_DataFrame & frame);
+
+  PBoolean IsValidRTPPayload(PBoolean muxed = true) const;
+  PBoolean IsNotMultiplexed() const;
+
+};
+
+H323LIST(RTP_MultiDataFrameList, RTP_MultiDataFrame);
 
 
 /**An RTP control frame encapsulation.
@@ -521,6 +313,32 @@ class RTP_ControlFrame : public PBYTEArray
     PINDEX compoundSize;
 };
 
+/**This class is for encapsulating the Multiplexing of RTCP.
+ */
+
+class RTP_MultiControlFrame : public PBYTEArray
+{
+  PCLASSINFO(RTP_MultiControlFrame, PBYTEArray);
+
+ public:
+
+  RTP_MultiControlFrame(
+      BYTE const * buffer,   ///< Pointer to an array of BYTEs.
+      PINDEX length          ///< Number of elements pointed to by \p buffer.
+  );
+
+  RTP_MultiControlFrame(
+      PINDEX rtplen          ///< Length of RTP Frame
+  );
+
+  int  GetMultiHeaderSize() const;
+  WORD GetMultiplexID() const;
+  void SetMultiplexID(WORD id);
+  void GetRTCPPayload(RTP_ControlFrame & frame) const;
+  void SetRTCPPayload(RTP_ControlFrame & frame);
+
+};
+
 
 /**This class is for encpsulating the IETF Real Time Protocol interface.
  */
@@ -535,8 +353,9 @@ class RTP_Session : public PObject
       DefaultAudioSessionID    = 1,
       DefaultVideoSessionID    = 2,
       DefaultFaxSessionID      = 3,
-	  DefaultH224SessionID     = 4,
-	  DefaultExtVideoSessionID = 5
+      DefaultH224SessionID     = 3,
+      DefaultExtVideoSessionID = 4,
+      DefaultFileSessionID     = 5
     };
 
   /**@name Construction */
@@ -612,6 +431,12 @@ class RTP_Session : public PObject
         int & selectStatus
      );
 
+    /**Check the data before sending to the RTP Channel
+      */
+    virtual PBoolean PreWriteData(
+        RTP_DataFrame & frame   ///<  Frame to write to the RTP session
+    ) = 0;
+
     /**Write a data frame from the RTP channel.
       */
     virtual PBoolean WriteData(
@@ -671,10 +496,11 @@ class RTP_Session : public PObject
         void PrintOn(ostream &) const;
 
         DWORD sourceIdentifier;
-        PTime realTimestamp;
-        DWORD rtpTimestamp;
-        DWORD packetsSent;
-        DWORD octetsSent;
+        PTime  realTimestamp;
+        PInt64 realTimestamp1970;
+        DWORD  rtpTimestamp;
+        DWORD  packetsSent;
+        DWORD  octetsSent;
     };
     virtual void OnRxSenderReport(const SenderReport & sender,
                                   const ReceiverReportArray & reports);
@@ -922,6 +748,8 @@ class RTP_Session : public PObject
 #endif
 
     PBoolean          ignoreOtherSources;
+    unsigned          ignoreOtherSourcesCount;
+    unsigned          ignoreOtherSourceMaximum;
     PBoolean          ignoreOutOfOrderPackets;
     DWORD         syncSourceOut;
     DWORD         syncSourceIn;
@@ -1186,6 +1014,10 @@ class RTP_UDP : public RTP_Session
       */
     virtual PBoolean PseudoRead(int & selectStatus);
 
+    /**Check the data before sending to the RTP Channel
+      */
+    PBoolean PreWriteData(RTP_DataFrame & frame);
+
     /**Write a data frame from the RTP channel.
       */
     virtual PBoolean WriteData(RTP_DataFrame & frame);
@@ -1340,6 +1172,7 @@ class RTP_UDP : public RTP_Session
     PBoolean enableGQOS;
 
     PBoolean remoteIsNAT;
+    unsigned successiveWrongAddresses;
 };
 
 
