@@ -472,6 +472,10 @@ class H323Transport : public PIndirectChannel
       H323RasPDU & pdu,                     ///<  GatekeeperRequest PDU
       const H323TransportAddress & address  ///<  Address of gatekeeper (if present)
     );
+
+   /**Whether the RAS is tunnelled AKA H.460.17
+      */
+    virtual PBoolean IsRASTunnelled()  { return false; }
   //@}
 
 
@@ -490,6 +494,11 @@ class H323Transport : public PIndirectChannel
     /**Wait for associated thread to terminate.
       */
     void CleanUpOnTermination();
+
+    /**Get Error Code
+       Default calls PChannel::GetErrorCode(ErrorGroup group)
+      */
+    virtual PChannel::Errors GetErrorCode(ErrorGroup group = NumErrorGroups) const;
   //@}
 
   protected:
